from setuptools import setup, Extension, find_packages
import sys, os, subprocess, shutil

pjoin=os.path.join
dirname=os.path.dirname
normpath = os.path.normpath
isfile = os.path.isfile
isdir = os.path.isdir
verbose=int(os.environ.get("SETUP_VERBOSE","0"))
limited_abi = int(os.environ.get('LIMITED_ABI','38'))

def lineList(L):
    return '\n     '+('\n     '.join((repr(_) for _ in L)))

def lineListDir(d):
    return lineList(os.listdir(d))

cmdclass={}
install_requires = []
ext_modules=[]
data_files = []
setup_py = sys.argv[0]=='setup.py'
if setup_py and 'help' in sys.argv:
    print('''
    python setup.py clean remove fribidi-src, build & dist
    python setup.py help  to give this help
    python setup.py sdist to make a source distro
    pip wheel -w dist . [-v] to make a wheel
''')
    sys.exit(0)
elif setup_py and 'clean' in sys.argv:
    for d in 'fribidi-src build dist'.split():
        d = pjoin(dirname(__file__),d)
        if isdir(d):
            if verbose: print(f'##### shutil.rmtree({d!r})')
            shutil.rmtree(d)
    sys.exit(0)
elif setup_py and 'sdist' in sys.argv:
    data_files = [pjoin("src","_pyfribidi.c")]
else:
    py_limited_kwds = {}
    if limited_abi:
        #+++++++++++++++++++++++++ start limited C api support
        try:
            from setuptools.command.bdist_wheel import bdist_wheel, get_abi_tag
        except ImportError:
            try:
                from wheel.bdist_wheel import bdist_wheel, get_abi_tag
            except ImportError:
                from wheel._bdist_wheel import bdist_wheel, get_abi_tag
        def make_la_info():
            '''compute limited api and abi info'''
            global py_limited_kwds, cpstr
            cpstr = get_abi_tag()
            if cpstr.startswith("cp"):
                lav = '0x03080000'
                cpstr = f'cp{limited_abi}'
                if sys.platform == "darwin":
                    machine = sysconfig.get_platform().split('-')[-1]
                    if machine=='arm64' or os.environ.get('ARCHFLAGS','')=='-arch arm64':
                        #according to cibuildwheel/github M1 supports pythons >= 3.8
                        lav = '0x03080000'
                        cpstr = f'cp{max(limited_abi,38)}'
                py_limited_kwds = dict(
                                        define_macros=[("Py_LIMITED_API", lav)],
                                        py_limited_api=True,
                                        )

        make_la_info()

        class bdist_wheel_abi3(bdist_wheel):
            def get_tag(self):
                python, abi, plat = super().get_tag()

                if python.startswith("cp"):
                    abi = 'abi3'
                    python = cpstr
                return python,abi,plat
        cmdclass={"bdist_wheel": bdist_wheel_abi3}
    #------------------------- end   limited C api support

    def locationValueError(msg):
        print('!!!!! %s\nls(%r)\n%s\n!!!!!''' % (msg,cwd,lineListDir(cwd)))
        raise ValueError(msg)

    def sprun(args):
        print(f'##### about to execute\n  {" ".join(args)}')
        try:
            subprocess.run(args)
        except:
            t,e,b = sys.exc_info()
            print(f'!!!!! {args[0]} raised {e}')
            raise

    def setupFribidiSrc(target):
        from dulwich import porcelain
        install_requires.extend(['dulwich','meson','ninja'])
        porcelain.clone("https://github.com/fribidi/fribidi", target,
                  refspecs=[b'cfc71cda065db859d8b4f1e3c6fe5da7ab02469a'])
        cwd = os.getcwd()
        os.chdir(target)
        try:
            sprun(['meson','setup','-Ddocs=false','--backend=ninja','build'])
            sprun(['ninja','-C','build','test'])
        finally:
            os.chdir(cwd)

    def getFribidiSrc():
        print(f'##### attempting git clone an meson/ninja build in {os.getcwd()}')
        try:
            target = 'fribidi-src'
            if os.path.isdir(target):
                if int(os.environ.get('CLEAN_FRIBIDI','0'))>=1:
                    shutil.rmtree(target)
                    print(f'##### removed existing directory {target!r}')
                    setupFribidiSrc(target)
                else:
                    print(f'##### using existing directory {target!r}')
            else:
                setupFribidiSrc(target)
        except:
            t,e,b = sys.exc_info()
            print(f'!!!!! clone and build commands failed with {e}')
            raise
        else:
            return target

    fribidi_src = getFribidiSrc()
    pyfribidi_src = 'src'
    if verbose:
        print("+++++ fribidi_src=%r\n+++++ pyfribidi_src=%r" % (fribidi_src,pyfribidi_src))

    meson_lib = pjoin(fribidi_src,'build','lib')
    def getIncludeDirs():
        for _top in ('build',None):
            top = pjoin(fribidi_src,_top) if _top else fribidi_src
            lib = pjoin(top,'lib')
            if isfile(pjoin(top,'config.h')) and isfile(pjoin(lib,'fribidi-config.h')):
                I = [top,lib]
                if _top:
                    gen = pjoin(top,'gen.tab')
                    if isfile(pjoin(gen,'fribidi-unicode-version.h')):
                        I.append(gen)
                    return I
        locationValueError('''Cannot locate a suitable config.h file.
        meson setup -Ddocs=false --backend=ninja build
        ninja -C build test
    or
        ./autogen.sh
        ./configure''')
    include_dirs = getIncludeDirs() + [pjoin(fribidi_src,"lib"),pjoin(fribidi_src,'gen.tab'),pyfribidi_src]
    if verbose:
        print("+++++ include_dirs=%s" % lineList(include_dirs))

    if isdir(meson_lib):
        if sys.platform=='win32':
            if verbose:
                print('+++++ meson_lib ls(%r)\n%s' % (meson_lib,lineListDir(meson_lib)))
            meson_lib = pjoin(meson_lib,'fribidi.lib')
        else:
            meson_lib = pjoin(meson_lib,'libfribidi.a')

        if not isfile(meson_lib):
            meson_lib = None
    else:
        meson_lib = None


    libraries = []
    if meson_lib:
        extra_objects = [meson_lib]
        lib_sources = []
        if verbose:
            print('+++++ using static libraries %s' % lineList(libraries))
    else:
        extra_objects = []
        lib_sources = [pjoin(fribidi_src,p) for p in """
                        lib/fribidi.c
                        lib/fribidi-arabic.c
                        lib/fribidi-bidi.c
                        lib/fribidi-bidi-types.c
                        lib/fribidi-brackets.c
                        lib/fribidi-deprecated.c
                        lib/fribidi-joining.c
                        lib/fribidi-joining-types.c
                        lib/fribidi-mirroring.c
                        lib/fribidi-run.c
                        lib/fribidi-shape.c
                        lib/fribidi-char-sets-cp1256.c
                        lib/fribidi-char-sets-iso8859-8.c
                        lib/fribidi-char-sets-cap-rtl.c
                        lib/fribidi-char-sets-utf8.c
                        lib/fribidi-char-sets.c
                        lib/fribidi-char-sets-cp1255.c
                        lib/fribidi-char-sets-iso8859-6.c
                        """.split()]
    define_macros = [("HAVE_CONFIG_H", 1)] + py_limited_kwds.pop('define_macros',[])
    ext_modules=[
        Extension(
            name='pyfribidi._pyfribidi',
            sources=[pjoin("src",'_pyfribidi.c')] + lib_sources,
            define_macros=define_macros,
            libraries=libraries,
            extra_objects = extra_objects,
            include_dirs=include_dirs,
            **py_limited_kwds,
            ),
        ]

def get_version():
    with open(pjoin("src","pyfribidi","__init__.py"),"r") as f:
        for line in f.readlines():
            line = line.strip()
            if line.startswith('__version__'):
                version = eval(line.split('=')[1].strip(),{})

    if ext_modules:
        with open(pjoin("src","pyfribidi_version.h"),'w') as f:
            f.write('#define PYFRIBIDI_VERSION %s\n' % version)

    return version

setup(
    name="pyfribidi",
    version=get_version(),
    ext_modules = ext_modules,
    author="Yaacov Zamir, Nir Soffer, Robin Becker",
    author_email="kzamir@walla.co.il",
    long_description = open("README.rst").read(),
    packages = find_packages("src"),
    package_dir = {'': "src"},
    data_files = data_files,
    install_requires = install_requires,
    extras_require={},
    cmdclass = cmdclass,
)



"""simple Python binding for fribidi.

pyfribidi uses libfribidi to order text visually using the unicode
algorithm. pyfribidi can also convert text from visual order to
logical order, but the conversion may be wrong in certain cases.
"""
__version__ = '0.15.0'
__all__ = ('log2vis', 'LTR', 'ON', 'RTL', 'WLTR', 'WRTL', 'pyFribidiVersion', 'fribidiVersion', 'fribidiInterfaceVersion', 'fribidiUnicodeVersion', '_log2vis', 'log2vis')
import sys
isPy3 = sys.version_info[0]==3
if isPy3:
    unicode = str

from . _pyfribidi import LTR, ON, RTL, WLTR, WRTL, pyFribidiVersion, fribidiVersion, fribidiInterfaceVersion, fribidiUnicodeVersion, log2vis as _log2vis
assert __version__==pyFribidiVersion, "Non matching version pyfribidi=%s!= _pyfribidi=%s" % (__version__,pyFribidiVersion)

def log2vis(logical, base_direction=RTL, encoding="utf-8", clean=False, reordernsm=True):
    """
    Return string reordered visually according to base direction.
    Return the same type of input string, either unicode or string using
    encoding.

    Note that this function does not handle line breaking. You should
    call log2vis with each line.

    Arguments:
    - logical: unicode or encoded string
    - base_direction: optional logical base direction. Accepts one of
      the constants LTR, RTL or ON, defined in this module. ON calculate
      the base direction according to the BiDi algorithm.
    - encoding: optional string encoding (ignored for unicode input)
    """

    if not isinstance(logical, unicode):
        logical = unicode(logical, encoding)
    else:
        encoding = None
    res = _log2vis(logical, base_direction=base_direction, clean=clean, reordernsm=reordernsm)
    if encoding:
        return res.encode(encoding)
    return res



#!/usr/bin/python
# -*- coding: utf-8 -*-
'''
These are very basic tests, because fribidi has its own tests. It may be
better to reuse fribidi own tests, but its not clear what is the value
of base_dir for those tests.
'''

import sys
import unittest
import pyfribidi
from pyfribidi import RTL, LTR, ON

def U(b):
    return b.decode('utf8')

class TestSkipped(Exception):
    '''Raised when test can not run'''

class InputTests(unittest.TestCase):

    def testRequireInputString(self):
        '''input: require input string or unicode'''
        self.assertRaises(TypeError, pyfribidi.log2vis)

    def testInvalidInputString(self):
        '''input: raise TypeError for non string or unicode input'''
        self.assertRaises(TypeError, pyfribidi.log2vis, 1)

    def testInvalidDirection(self):
        '''input: raise ValueError for invalid direction'''
        self.assertRaises(ValueError, pyfribidi.log2vis, U(b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d'),
                          base_direction=1)

    def testUnknownEncoding(self):
        '''input: raise LookupError for invalid encoding'''
        self.assertRaises(LookupError, pyfribidi.log2vis, b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d',
                          encoding='foo')

    def testInvalidEncodedString(self):
        '''input: raise UnicodeError for invalid encoded string'''
        self.assertRaises(UnicodeError, pyfribidi.log2vis, b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d',
                          encoding='iso8859-8')

class UnicodeTests(unittest.TestCase):

    def testEmpty(self):
        '''unicode: empty string'''
        self.assertEqual(pyfribidi.log2vis(U(b'')), U(b''))

    def testBigString(self):
        '''unicode: big string

        It does not make sense to order such big strings, this just
        checks that there are no size limits in pyfribidi.
        '''
        # About 2MB string for default python build (ucs2)
        big = (U(b'\xd7\x90') * 1024) * 1024
        self.assertEqual(pyfribidi.log2vis(big), big)

    def testDefaultDirection(self):
        '''unicode: use RTL default'''
        self.assertEqual(pyfribidi.log2vis(U(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d')),
                         pyfribidi.log2vis(U(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d'), RTL))

    def testAsRTL(self):
        '''unicode: reorder line as RTL'''
        self.assertEqual(pyfribidi.log2vis(U(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d'), RTL),
                         U(b'\xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9 - hello'))

    def testAsLTR(self):
        '''unicode: reorder line as LTR'''
        self.assertEqual(pyfribidi.log2vis(U(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d'), LTR),
                         U(b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9'))

    def testNaturalLTR(self):
        '''unicode: reorder LTR line by natural order'''
        self.assertEqual(pyfribidi.log2vis(U(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d'), ON),
                         U(b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9'))

    def testNaturalRTL(self):
        '''unicode: reorder RTL line by natural order'''
        self.assertEqual(pyfribidi.log2vis(U(b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d - hello'), ON),
                         U(b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9'))

    def testNoReorderNonSpacingMarks(self):
        '''unicode: reorder non spacing marks'''
        #self.assertEqual(pyfribidi.log2vis(U(b'\xd7\x97\xd6\xb7\xd7\x99\xd6\xb0\xd7\xa4\xd6\xb7\xd7\x90'), RTL, reordernsm=False),
        #                U(b'\xd7\x90\xd6\xb7\xd7\xa4\xd6\xb0\xd7\x99\xd6\xb7\xd7\x97')
        #                )
        self.assertEqual(pyfribidi.log2vis(U(b'\xd8\xb5\xd9\x90\xd8\xb1\xd9\x8e\xd8\xa7\xd8\xb7\xd9\x8e \xd8\xa7\xd9\x84\xd9\x91\xd9\x8e\xd8\xb0\xd9\x90\xd9\x8a\xd9\x86\xd9\x8e \xd8\xa7\xd9\x8e\xd9\x86\xd9\x92\xd8\xb9\xd9\x8e\xd9\x85\xd9\x92\xd8\xaa\xd9\x8e \xd8\xb9\xd9\x8e\xd9\x84\xd9\x8e\xd9\x8a\xd9\x87\xd9\x90\xd9\x85\xd9\x92 \xd8\xba\xd9\x8e\xd9\x8a\xd9\x92\xd8\xb1\xd9\x90 \xd8\xa7\xd9\x84\xd9\x92\xd9\x85\xd9\x8e\xd8\xba\xd9\x92\xd8\xb6\xd9\x8f\xd9\x88\xd8\xa8\xd9\x90 \xd8\xb9\xd9\x8e\xd9\x84\xd9\x8e\xd9\x8a\xd9\x92\xd9\x87\xd9\x90\xd9\x85 \xd9\x88\xd9\x8e \xd9\x84\xd9\x8e\xd8\xa7 \xd8\xa7\xd9\x84\xd8\xb6\xd9\x91\xd9\x8e\xd9\x93\xd8\xa7\xd9\x84\xd9\x91\xd9\x90\xd9\x8a\xd9\x86\xd9\x8e'), RTL, reordernsm=False),
                        U(b'\xef\xbb\xa6\xd9\x8e\xef\xbb\xb4\xef\xbb\x9f\xd9\x91\xd9\x90\xef\xba\x8e\xef\xbb\x80\xd9\x91\xd9\x8e\xd9\x93\xef\xbb\x9f\xef\xba\x8d \xef\xba\x8e\xef\xbb\x9f\xd9\x8e \xef\xbb\xad\xd9\x8e \xef\xbb\xa2\xef\xbb\xac\xd9\x90\xef\xbb\xb4\xd9\x92\xef\xbb\xa0\xd9\x8e\xef\xbb\x8b\xd9\x8e \xef\xba\x8f\xd9\x90\xef\xbb\xae\xef\xbb\x80\xd9\x8f\xef\xbb\x90\xd9\x92\xef\xbb\xa4\xd9\x8e\xef\xbb\x9f\xd9\x92\xef\xba\x8d \xef\xba\xae\xd9\x90\xef\xbb\xb4\xd9\x92\xef\xbb\x8f\xd9\x8e \xef\xbb\xa2\xd9\x92\xef\xbb\xac\xd9\x90\xef\xbb\xb4\xef\xbb\xa0\xd9\x8e\xef\xbb\x8b\xd9\x8e \xef\xba\x96\xd9\x8e\xef\xbb\xa4\xd9\x92\xef\xbb\x8c\xd9\x8e\xef\xbb\xa7\xd9\x92\xef\xba\x8d\xd9\x8e \xef\xbb\xa6\xd9\x8e\xef\xbb\xb3\xef\xba\xac\xd9\x90\xef\xbb\x9f\xd9\x91\xd9\x8e\xef\xba\x8d \xef\xbb\x81\xd9\x8e\xef\xba\x8d\xef\xba\xae\xd9\x8e\xef\xba\xbb\xd9\x90')
                        )

    def testReorderNonSpacingMarks(self):
        '''unicode: reorder non spacing marks'''
        self.assertEqual(pyfribidi.log2vis(U(b'\xd7\x97\xd6\xb7\xd7\x99\xd6\xb0\xd7\xa4\xd6\xb7\xd7\x90'), RTL),
                         U(b'\xd7\x90\xd7\xa4\xd6\xb7\xd7\x99\xd6\xb0\xd7\x97\xd6\xb7')
                         )

class UTF8Tests(unittest.TestCase):
    '''Same tests for utf8, used mainly on linux'''

    def testEmpty(self):
        '''utf8: empty string'''
        self.assertEqual(pyfribidi.log2vis(''), '')

    def testBigString(self):
        '''utf8: big string

        It does not make sense to order such big strings, this just
        checks that there are no size limits in pyfribidi.
        '''
        # About 2MB string
        big = ('א' * 1024) * 1024
        self.assertEqual(pyfribidi.log2vis(big), big)

    def testDefaultDirection(self):
        '''utf8: use RTL default'''
        self.assertEqual(pyfribidi.log2vis(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d'),
                         pyfribidi.log2vis(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d', RTL))

    def testAsRTL(self):
        '''utf8: reorder line as RTL'''
        self.assertEqual(pyfribidi.log2vis(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d', RTL),
                         b'\xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9 - hello')

    def testAsLTR(self):
        '''utf8: reorder line as LTR'''
        self.assertEqual(pyfribidi.log2vis(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d', LTR),
                         b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9')

    def testNaturalLTR(self):
        '''utf8: reorder LTR line by natural order'''
        self.assertEqual(pyfribidi.log2vis(b'hello - \xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d', ON),
                         b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9')

    def testNaturalRTL(self):
        '''utf8: reorder RTL line by natural order'''
        self.assertEqual(pyfribidi.log2vis(b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d - hello', ON),
                         b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9')

    def testNoReorderNonSpacingMarks(self):
        '''utf8: reorder non spacing marks'''
        #self.assertEqual(pyfribidi.log2vis(b'\xd7\x97\xd6\xb7\xd7\x99\xd6\xb0\xd7\xa4\xd6\xb7\xd7\x90', RTL, reordernsm=False),
        #                b'\xd7\x90\xd6\xb7\xd7\xa4\xd6\xb0\xd7\x99\xd6\xb7\xd7\x97'
        #                )
        self.assertEqual(pyfribidi.log2vis(b'\xd8\xb5\xd9\x90\xd8\xb1\xd9\x8e\xd8\xa7\xd8\xb7\xd9\x8e \xd8\xa7\xd9\x84\xd9\x91\xd9\x8e\xd8\xb0\xd9\x90\xd9\x8a\xd9\x86\xd9\x8e \xd8\xa7\xd9\x8e\xd9\x86\xd9\x92\xd8\xb9\xd9\x8e\xd9\x85\xd9\x92\xd8\xaa\xd9\x8e \xd8\xb9\xd9\x8e\xd9\x84\xd9\x8e\xd9\x8a\xd9\x87\xd9\x90\xd9\x85\xd9\x92 \xd8\xba\xd9\x8e\xd9\x8a\xd9\x92\xd8\xb1\xd9\x90 \xd8\xa7\xd9\x84\xd9\x92\xd9\x85\xd9\x8e\xd8\xba\xd9\x92\xd8\xb6\xd9\x8f\xd9\x88\xd8\xa8\xd9\x90 \xd8\xb9\xd9\x8e\xd9\x84\xd9\x8e\xd9\x8a\xd9\x92\xd9\x87\xd9\x90\xd9\x85 \xd9\x88\xd9\x8e \xd9\x84\xd9\x8e\xd8\xa7 \xd8\xa7\xd9\x84\xd8\xb6\xd9\x91\xd9\x8e\xd9\x93\xd8\xa7\xd9\x84\xd9\x91\xd9\x90\xd9\x8a\xd9\x86\xd9\x8e', RTL, reordernsm=False),
                        b'\xef\xbb\xa6\xd9\x8e\xef\xbb\xb4\xef\xbb\x9f\xd9\x91\xd9\x90\xef\xba\x8e\xef\xbb\x80\xd9\x91\xd9\x8e\xd9\x93\xef\xbb\x9f\xef\xba\x8d \xef\xba\x8e\xef\xbb\x9f\xd9\x8e \xef\xbb\xad\xd9\x8e \xef\xbb\xa2\xef\xbb\xac\xd9\x90\xef\xbb\xb4\xd9\x92\xef\xbb\xa0\xd9\x8e\xef\xbb\x8b\xd9\x8e \xef\xba\x8f\xd9\x90\xef\xbb\xae\xef\xbb\x80\xd9\x8f\xef\xbb\x90\xd9\x92\xef\xbb\xa4\xd9\x8e\xef\xbb\x9f\xd9\x92\xef\xba\x8d \xef\xba\xae\xd9\x90\xef\xbb\xb4\xd9\x92\xef\xbb\x8f\xd9\x8e \xef\xbb\xa2\xd9\x92\xef\xbb\xac\xd9\x90\xef\xbb\xb4\xef\xbb\xa0\xd9\x8e\xef\xbb\x8b\xd9\x8e \xef\xba\x96\xd9\x8e\xef\xbb\xa4\xd9\x92\xef\xbb\x8c\xd9\x8e\xef\xbb\xa7\xd9\x92\xef\xba\x8d\xd9\x8e \xef\xbb\xa6\xd9\x8e\xef\xbb\xb3\xef\xba\xac\xd9\x90\xef\xbb\x9f\xd9\x91\xd9\x8e\xef\xba\x8d \xef\xbb\x81\xd9\x8e\xef\xba\x8d\xef\xba\xae\xd9\x8e\xef\xba\xbb\xd9\x90'
                        )

    def testReorderNonSpacingMarks(self):
        '''unicode: reorder non spacing marks'''
        self.assertEqual(pyfribidi.log2vis(b'\xd7\x97\xd6\xb7\xd7\x99\xd6\xb0\xd7\xa4\xd6\xb7\xd7\x90', RTL),
                         b'\xd7\x90\xd7\xa4\xd6\xb7\xd7\x99\xd6\xb0\xd7\x97\xd6\xb7'
                         )

class OtherEncodingsTests(unittest.TestCase):
    '''Minimal tests for other encodings'''

    def testIso8859_8NaturalRTL(self):
        '''other encodings: iso8859-8'''
        charset = 'iso8859-8'
        self.assertEqual(pyfribidi.log2vis(U(b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d - hello').encode(charset),
                                           encoding=charset),
                         U(b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9').encode(charset))

    def testCp1255NaturalRTL(self):
        '''other encodings: cp1255'''
        charset = 'cp1255'
        self.assertEqual(pyfribidi.log2vis(U(b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d - hello').encode(charset),
                                           encoding=charset),
                         U(b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9').encode(charset))

    def testUTF16NaturalRTL(self):
        '''other encodings: utf-16'''
        charset = 'utf-16'
        self.assertEqual(pyfribidi.log2vis(U(b'\xd7\xa9\xd7\x9c\xd7\x95\xd7\x9d - hello').encode(charset),
                                           encoding=charset),
                         U(b'hello - \xd7\x9d\xd7\x95\xd7\x9c\xd7\xa9').encode(charset))

class Crasher(unittest.TestCase):
    def test_glibc_free_invalid_next_size(self):
        # *** glibc detected *** /home/ralf/py27/bin/python2: free(): invalid next size (fast): 0x00000000011cff00 ***
        pyfribidi.log2vis(b'\\xf0\\x90\\x8e\\xa2\\xf0\\x90\\x8e\\xaf\\xf0\\x90\\x8e\\xb4\\xf0\\x90\\x8e\\xa1\\xf0\\x90\\x8f\\x83')

if __name__ == '__main__':
    suite = unittest.defaultTestLoader.loadTestsFromName(__name__)
    res = unittest.TextTestRunner().run(suite)
    sys.exit(not res.wasSuccessful())
